﻿using MigraDoc.DocumentObjectModel;
using MigraDoc.Rendering;
using CustomComponents.Helpers;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CustomComponents.NonVisualComponents
{
    public partial class PdfImage : Component
    {
        public PdfImage()
        {
            InitializeComponent();
        }

        public PdfImage(IContainer container)
        {
            container.Add(this);

            InitializeComponent();
        }
        public bool CreatePdfDoc(DataForImage dataForImage)
        {
            Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);
            CheckFileExsists(dataForImage);
          
            Document _document = new Document();
            var style = _document.Styles["Normal"];
            style.Font.Name = "Arial";
            style.Font.Size = 25;
            style = _document.Styles.AddStyle("NormalTitle", "Normal");
            style.Font.Bold = true;

          
            var section = _document.AddSection();
            var paragraph = section.AddParagraph(dataForImage.header);
            paragraph.Format.Alignment = ParagraphAlignment.Center;
            paragraph.Format.SpaceAfter = "2cm";

        
            foreach (string path in dataForImage.imagepaths)
                if (File.Exists(path)) section.AddImage(path);

            
            PdfDocumentRenderer renderer = new PdfDocumentRenderer(true);
            renderer.Document = _document;
            renderer.RenderDocument();
            renderer.PdfDocument.Save(dataForImage.filepath);

            return true;
        }
        private void CheckFileExsists(DataForImage dataForImage)
        {
            if (string.IsNullOrEmpty(dataForImage.filepath) || string.IsNullOrEmpty(dataForImage.header) || dataForImage.imagepaths.Length == 0)
            {
                throw new ArgumentNullException();
            }
            if (!File.Exists(dataForImage.filepath))
            {
                throw new FileNotFoundException(dataForImage.filepath);
            }
        }
    }
}
